import re

from netmiko.cisco_base_connection import CiscoSSHConnection


class MypowerOsSSH(CiscoSSHConnection):

    def session_preparation(self) -> None:
        """Prepare the session after the connection has been established."""
        self.ansi_escape_codes = True
        # 根据账号权限不同，进入后显示的prompt也会有差异
        self._test_channel_read(pattern=r">|#")
        # 迈普设备需要enable 才能取消分页，或者账号权限足够
        if self.secret:
            self.enable()
            self.disable_paging(command="more off")
            self.exit_enable_mode()
            self.set_base_prompt()
        else:
            self.disable_paging(command="more off")

    # 迈普的设备会有两种符号，类似思科，一种是# ，一种是>
    def set_base_prompt(self, pri_prompt_terminator=r'#',
                        alt_prompt_terminator=r'>',
                        delay_factor=0.1) -> str:
        return super(MypowerOsSSH, self).set_base_prompt(pri_prompt_terminator=pri_prompt_terminator,
                                                         alt_prompt_terminator=alt_prompt_terminator,
                                                         delay_factor=delay_factor)

    def normalize_linefeeds(self, a_string: str) -> str:
        """Convert '\r\n' or '\r\r\n' to '\n, and remove extra '\r's in the text."""
        newline = re.compile(r"(\r\r\n\r|\r\r\n|\r\n)")
        return newline.sub(self.RESPONSE_RETURN, a_string).replace("\r", "\n")

    def enable(
        self,
        cmd="enable",
        pattern="ssword",
        enable_pattern=None,
        re_flags=re.IGNORECASE,
    ):
        """Enter enable mode."""
        return super().enable(
            cmd=cmd, pattern=pattern, enable_pattern=enable_pattern, re_flags=re_flags
        )

    def check_enable_mode(self, check_string="#"):
        return super().check_enable_mode(check_string=check_string)

    def check_config_mode(self, check_string: str = ")#", pattern: str = "") -> bool:
        """
        Checks if the device is in configuration mode or not.
        """
        return super().check_config_mode(check_string=check_string, pattern=pattern)

    # 迈普的离开enable 模式命令有disable 和exit 两种，直接使用原有的方法即可
    # def exit_enable_mode(self, exit_command="disable")

    def config_mode(
            self,
            config_command: str = "configure terminal",
            pattern: str = "",
            re_flags: int = 0,
    ) -> str:
        return super().config_mode(
            config_command=config_command, pattern=pattern, re_flags=re_flags
        )
    # 迈普的 exit_config_mode 参数与思科的基类完全一致，无需修改
    # def exit_config_mode(self, exit_config: str = "end", pattern: str = r"\#") -> str:
    #     """Exit from configuration mode."""
    #     return super().exit_config_mode(exit_config=exit_config, pattern=pattern)

    def save_config(
            self,
            cmd="write",
            confirm=True,
            confirm_response="yes",
    ):
        """Saves Config."""
        return super(MypowerOsSSH, self).save_config(
            cmd=cmd,
            confirm=confirm,
            confirm_response=confirm_response,
        )



from netmiko.ssh_dispatcher import CLASS_MAPPER,platforms,platforms_str,\
    telnet_platforms_str,CLASS_MAPPER_BASE,FILE_TRANSFER_MAP
from zetops.zetmiko.huawei import HuaweiZetSSH
from zetops.zetmiko.hillstone import HillStoneSSH
from zetops.zetmiko.fiberhome import FiberHomeSSH
from zetops.zetmiko.mypower import MypowerOsSSH

CLASS_MAPPER['hillstone'] = HillStoneSSH
CLASS_MAPPER['huawei'] = HuaweiZetSSH
CLASS_MAPPER['fiberhome'] = FiberHomeSSH
CLASS_MAPPER['mypower'] = MypowerOsSSH

platforms = list(CLASS_MAPPER.keys())
platforms.sort()
platforms_base = list(CLASS_MAPPER_BASE.keys())
platforms_base.sort()
platforms_str = "\n".join(platforms_base)
platforms_str = "\n" + platforms_str

scp_platforms = list(FILE_TRANSFER_MAP.keys())
scp_platforms.sort()
scp_platforms_str = "\n".join(scp_platforms)
scp_platforms_str = "\n" + scp_platforms_str

telnet_platforms = [x for x in platforms if "telnet" in x]
telnet_platforms_str = "\n".join(telnet_platforms)
telnet_platforms_str = "\n" + telnet_platforms_str


def ConnectHandler(*args, **kwargs):
    """Factory function selects the proper class and creates object based on device_type."""
    device_type = kwargs["device_type"]
    if device_type not in platforms:
        if device_type is None:
            msg_str = platforms_str
        else:
            msg_str = telnet_platforms_str if "telnet" in device_type else platforms_str
        raise ValueError(
            "Unsupported 'device_type' "
            "currently supported platforms are: {}".format(msg_str)
        )
    ConnectionClass = ssh_dispatcher(device_type)
    return ConnectionClass(*args, **kwargs)


Netmiko = ConnectHandler
Zetmiko = ConnectHandler


def ssh_dispatcher(device_type):
    """Select the class to be instantiated based on vendor/platform."""
    return CLASS_MAPPER[device_type]



import re

from netmiko.cisco_base_connection import CiscoSSHConnection


class HillStoneSSH(CiscoSSHConnection):
    def session_preparation(self) -> None:
        """Prepare the session after the connection has been established."""
        self.ansi_escape_codes = True
        self._test_channel_read(pattern=r"#")
        self.set_terminal_width(
            command="terminal width 511", pattern=r"terminal width 511"
        )
        self.disable_paging()
        self.set_base_prompt()
    # 山石设备没有目前来看只有一种符号"#" ，没有右括弧">"
    def set_base_prompt(self, pri_prompt_terminator=r'#',
                        alt_prompt_terminator=r'#',
                        delay_factor=0.1) -> str:
        return super(HillStoneSSH, self).set_base_prompt(pri_prompt_terminator=pri_prompt_terminator,
                                                         alt_prompt_terminator=alt_prompt_terminator,
                                                         delay_factor=delay_factor)

    def normalize_linefeeds(self, a_string: str) -> str:
        """Convert '\r\n' or '\r\r\n' to '\n, and remove extra '\r's in the text."""
        newline = re.compile(r"(\r\r\n\r|\r\r\n|\r\n)")
        return newline.sub(self.RESPONSE_RETURN, a_string).replace("\r", "\n")

    def check_enable_mode(self, check_string: str = "") -> bool:
        """Check if in enable mode. Return boolean."""
        return True

    def enable(
            self,
            cmd: str = "",
            pattern: str = "",
            enable_pattern= None,
            re_flags: int = re.IGNORECASE,
    ) -> str:
        """no enable mode."""
        return ""

    def exit_enable_mode(self, exit_command: str = "") -> str:
        """no enable (privileged exec) mode."""
        return ""

    # 目前来看进入config模式 形如config.*)的回显
    def check_config_mode(self, check_string: str = "config.*)#", pattern: str = "#") -> bool:
        """
        Checks if the device is in configuration mode or not.
        """
        return super().check_config_mode(check_string=check_string, pattern=pattern)

    def config_mode(
            self,
            config_command: str = "configure",
            pattern: str = "",
            re_flags: int = 0,
    ) -> str:
        return super().config_mode(
            config_command=config_command, pattern=pattern, re_flags=re_flags
        )

    def exit_config_mode(self, exit_config: str = "exit", pattern: str = r"#.*") -> str:
        """Exit from configuration mode."""
        return super().exit_config_mode(exit_config=exit_config, pattern=pattern)


    def save_config(self) -> str:
        """Saves Config Using save ,inputs y for save confirm,and inputs y for backup confirm"""
        save_cmd = 'save'
        self.enable()
        self.write_channel(save_cmd)
        self.write_channel(self.RETURN)
        save_output = self.read_until_prompt_or_pattern(pattern=r"\[y\]/n:")
        # first confirm
        self.write_channel('y')
        confirm_resp_1st = self.read_until_prompt_or_pattern(pattern=r"y/\[n\]:")
        # second confirm
        self.write_channel('y')
        confirm_resp_2nd = self.read_until_prompt_or_pattern(pattern=r"#")
        output = save_output + confirm_resp_1st + confirm_resp_2nd
        return output


import re

from netmiko.cisco_base_connection import CiscoSSHConnection
from netmiko.ssh_exception import NetmikoTimeoutException


class FiberHomeSSH(CiscoSSHConnection):

    def session_preparation(self) -> None:
        """Prepare the session after the connection has been established."""
        self.ansi_escape_codes = True
        self._test_channel_read(pattern=r"#|>")
        self.set_terminal_width(
            command="terminal width 511", pattern=r"terminal width 511"
        )
        self.disable_paging()
        self.set_base_prompt()

    def set_base_prompt(self, pri_prompt_terminator=r'#',
                        alt_prompt_terminator=r'>',
                        delay_factor=0.1) -> str:
        return super(FiberHomeSSH, self).set_base_prompt(pri_prompt_terminator=pri_prompt_terminator,
                                                         alt_prompt_terminator=alt_prompt_terminator,
                                                         delay_factor=delay_factor)

    def normalize_linefeeds(self, a_string: str) -> str:
        """Convert '\r\n' or '\r\r\n' to '\n, and remove extra '\r's in the text."""
        newline = re.compile(r"(\r\r\n\r|\r\r\n|\r\n)")
        return newline.sub(self.RESPONSE_RETURN, a_string).replace("\r", "\n")

    def send_command(
            self,
            command_string,
            expect_string=None,
            delay_factor=1,
            max_loops=500,
            auto_find_prompt=True,
            strip_prompt=True,
            strip_command=True,
            normalize=True,
            use_textfsm=False,
            textfsm_template=None,
            use_ttp=False,
            ttp_template=None,
            use_genie=False,
            cmd_verify=False,
    ):
        """
        烽火设备部分命令show的时候，回显在前，执行的命令在回显之后显示，顾将cmd_verify置为False
        """
        return super(FiberHomeSSH, self).send_command(command_string,
                                                      expect_string=expect_string,
                                                      delay_factor=delay_factor,
                                                      max_loops=max_loops,
                                                      auto_find_prompt=auto_find_prompt,
                                                      strip_prompt=strip_prompt,
                                                      strip_command=strip_command,
                                                      normalize=normalize,
                                                      use_textfsm=use_textfsm,
                                                      textfsm_template=textfsm_template,
                                                      use_ttp=use_ttp,
                                                      ttp_template=ttp_template,
                                                      use_genie=use_genie,
                                                      cmd_verify=cmd_verify)

    def send_command_timing(
            self,
            command_string,
            delay_factor=1,
            max_loops=150,
            strip_prompt=True,
            strip_command=True,
            normalize=True,
            use_textfsm=False,
            textfsm_template=None,
            use_ttp=False,
            ttp_template=None,
            use_genie=False,
            cmd_verify=False,
            cmd_echo=None,
    ):
        """
        烽火设备部分命令show的时候，回显在前，执行的命令在回显之后显示，顾将cmd_verify置为False
        """
        return super(FiberHomeSSH, self).send_command_timing(command_string,
                                                             delay_factor=delay_factor,
                                                             max_loops=max_loops,
                                                             strip_prompt=strip_prompt,
                                                             strip_command=strip_command,
                                                             normalize=normalize,
                                                             use_textfsm=use_textfsm,
                                                             textfsm_template=textfsm_template,
                                                             use_ttp=use_ttp,
                                                             ttp_template=ttp_template,
                                                             use_genie=use_genie,
                                                             cmd_verify=cmd_verify)

    def check_enable_mode(self, check_string: str = "#") -> bool:
        """Check if in enable mode. Return boolean."""
        return super(FiberHomeSSH, self).check_enable_mode(check_string=check_string)

    def enable(
            self,
            cmd: str = r"enable",
            pattern: str = "ssword",
            enable_pattern=r'#',
            re_flags: int = re.IGNORECASE,
    ) -> str:
        output = ""
        msg = (
            "Failed to enter enable mode. Please ensure you pass "
            "the 'secret' argument to ConnectHandler."
        )

        # Check if in enable mode
        # Send "enable" mode command
        self.write_channel(self.normalize_cmd(cmd))
        try:
            # Read the command echo
            end_data = ""
            if self.global_cmd_verify is not False:
                output += self.read_until_pattern(pattern=re.escape(cmd.strip()))
                end_data = output.split(cmd.strip())[-1]

            # Search for trailing prompt or password pattern
            if pattern not in output and self.base_prompt not in end_data:
                output += self.read_until_prompt_or_pattern(
                    pattern=pattern, re_flags=re_flags
                )
            # Send the "secret" in response to password pattern
            if re.search(pattern, output):
                self.write_channel(self.normalize_cmd(self.secret))
                output += self.read_until_prompt()

            # Search for terminating pattern if defined
            if enable_pattern and not re.search(enable_pattern, output):
                output += self.read_until_pattern(pattern=enable_pattern)
            else:
                if not self.check_enable_mode():
                    raise ValueError(msg)
        except NetmikoTimeoutException:
            raise ValueError(msg)
        return output

    def exit_enable_mode(self, exit_command: str = "end") -> str:
        return super(FiberHomeSSH, self).exit_enable_mode(exit_command=exit_command)

    # 目前来看进入config模式 形如config.*)的回显
    def check_config_mode(self, check_string: str = "config.*)#", pattern: str = "#") -> bool:
        """
        Checks if the device is in configuration mode or not.
        """
        return super(FiberHomeSSH, self).check_config_mode(check_string=check_string, pattern=pattern)

    def config_mode(
            self,
            config_command: str = "configure",
            pattern: str = "#",
            re_flags: int = 0,
    ) -> str:
        return super(FiberHomeSSH, self).config_mode(
            config_command=config_command, pattern=pattern, re_flags=re_flags
        )

    def exit_config_mode(self, exit_config: str = "end", pattern: str = r"#.*|>.*") -> str:
        """Exit from configuration mode."""
        return super().exit_config_mode(exit_config=exit_config, pattern=pattern)

    def save_config(self,
                    cmd="write file",
                    confirm=True,
                    confirm_response="y", ) -> str:
        return super(FiberHomeSSH, self).save_config(cmd=cmd,
                                                     confirm=confirm,
                                                     confirm_response=confirm_response)



import time
import re
from netmiko.huawei.huawei import HuaweiSSH
from netmiko.ssh_exception import NetmikoAuthenticationException
from netmiko import log

COMMIT_CMD = 'commit'
UNCOMMIT_FALG = ['Warning: Uncommitted configurations found']

EXIT_CMD = 'return'


class HuaweiZetSSH(HuaweiSSH):

    def commit(self):
        self.clear_buffer()
        cmd = f'{COMMIT_CMD}{self.RETURN}'
        self.write_channel(cmd)
        output = self.read_until_prompt_or_pattern(pattern=r"\[.*?\]")
        # Error: Unrecognized command found at '^' position
        if 'Unrecognized command' in output:
            raise Exception('This platform does not support commit,or the device is not in the config mode')

        return output

    def send_config_set(self, *args, **kwargs):
        '''
        如果自动commit则调用父类方法的时候，不能退出config模式，先调用重写的commit，
        然后根据用户最早传入的是否离开config模式决定是否离开config模式。
        如果非自动commit，暂时不离开config模式，根据回显判断是否有输入yes no的提示（询问是否不提交配置），自动输入一个yes
        :param auto_commit:  布尔 默认false ，是否自动commit
        :param kwargs:
        :return: 交互的整段回显
        '''
        intend_exit_config_mode = kwargs.get('exit_config_mode', True)
        # 执行结束后均不离开config模式 通过intend_exit_config_mode来执行
        kwargs['exit_config_mode'] = False
        if 'auto_commit' in kwargs:
            auto_commit = kwargs.get('auto_commit')
            del kwargs['auto_commit']
        else:
            auto_commit = False

        if auto_commit:

            output = super(HuaweiZetSSH, self).send_config_set(*args, **kwargs)
            output += self.commit()
            if intend_exit_config_mode:
                self.exit_config_mode()
        else:
            # Warning: Uncommitted configurations found. Are you sure to commit them before exiting? [Y(yes)/N(no)/C(cancel)]
            output = super(HuaweiZetSSH, self).send_config_set(*args, **kwargs)
            pattern = r'>|]|\[Y\S+/N\S+\]'
            if intend_exit_config_mode:
                self.clear_buffer()
                self.write_channel(f'{EXIT_CMD}{self.RETURN}')
                exit_output = self.read_until_prompt_or_pattern(pattern)
                output += exit_output
                # 如果提示有未提交的配置，则输入Y不提交配置。其他情况无此提示信息，则代表return执行成功
                if self._warning_uncommitted(exit_output):
                    pattern = '>'
                    self.write_channel('Y\n')
                    confirm_output = self.read_until_prompt_or_pattern(pattern)
                    output += confirm_output

        return output

    def _warning_uncommitted(self, output, uncommitted_flags=UNCOMMIT_FALG):
        for i in uncommitted_flags:
            if i in output:
                return True
        return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zetops',
 'zetops.zetmiko',
 'zetops.zetmiko.fiberhome',
 'zetops.zetmiko.hillstone',
 'zetops.zetmiko.huawei',
 'zetops.zetmiko.mypower']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'zetops',
    'version': '0.1.8',
    'description': 'A NetDevOps package aiming to improve NetDevOps(network automation) development efficiency for network development engineer!',
    'long_description': '\n# ZetOps\nZetOps是旨在提高网络运维自动化开发效率的一款工具包。\n\n名字的来源主要是之前做过的一个项目，叫做“织网”，\n\n你可以把z看做是“自”，它旨在解决网络自动化开发效率问题，垂直于网络自动化开发领域。\n\n你可以把Z看做是"织"，代表的是网络自动化人一针一线缝缝补补编织梦想的精神。\n\n你可以把Z看做是“中”,它的一大特色是应对国产化趋势之下国内网络设备适配的不足。\n\n你可以把Z看做是“N”的转置，代表的是网络人锐意创新的决心。\n\n然而，目前它还刚刚起步。\n\n# RoadMap\n\n1. 在这个最初的版本，我会聚焦在netmiko对国产化设备适配的不足，写一些国产化设备的驱动。\n\n对华为、山石、烽火进行了适配。\n\n2. 后续将会分享一些自己写的国产化textfsm的模板，也会试图寻求网友的帮助。\n\n3. 我会根据自己的理解，创建一个基于CLI的类似napalm的网络抽象层，不过这块可大可小，可能还需要我再思考一下。\n\n4. 同时也会针对NetDevOps场景中的数据提取和配置备份做一些nornir的task模块\n',
    'author': 'feifeiflight',
    'author_email': 'feifeiflight@126.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/jiujing/zetops',
    'packages': packages,
    'package_data': package_data,
}


setup(**setup_kwargs)
